@@ -351,7 +351,7 @@ static void sys_dispatchnextmidiin(void)
         else if (parserp->mp_status < MIDI_NOTEOFF)
         {
             /* running status w/out prev status byte or other invalid message */
-            error("dropping unexpected midi byte %02X", byte);
+            error("dropping unexpected MIDI byte %02X", byte);
         }
         else
         {
@@ -634,7 +634,7 @@ void sys_listmididevs(void)
         MAXNDEV, DEVDESCSIZE);
 
     if (!nindevs)
-        post("no midi input devices found");
+        post("no MIDI input devices found");
     else
     {
         post("MIDI input devices:");
@@ -642,7 +642,7 @@ void sys_listmididevs(void)
             post("%d. %s", i+1, indevlist + i * DEVDESCSIZE);
     }
     if (!noutdevs)
-        post("no midi output devices found");
+        post("no MIDI output devices found");
     else
     {
         post("MIDI output devices:");
@@ -662,7 +662,7 @@ void sys_set_midi_api(int which)
 #endif
     default:
         if (sys_verbose)
-            post("Ignoring unknown MIDI-API %d", which);
+            post("ignoring unknown MIDI API %d", which);
         return;
     }
 
