@@ -118,7 +118,7 @@ struct ex_ex * ex_if(t_expr *expr, struct ex_ex *argv, struct ex_ex *optr,
 static void ex_ldexp(t_expr *expr, long argc, struct ex_ex *argv, struct ex_ex *optr);
 static void ex_imodf(t_expr *expr, long argc, struct ex_ex *argv, struct ex_ex *optr);
 static void ex_modf(t_expr *expr, long argc, struct ex_ex *argv, struct ex_ex *optr);
-#if !defined(_MSC_VER) || (_MSC_VER >= 17000)
+#if !defined(_MSC_VER) || (_MSC_VER >= 1700)
 static void ex_cbrt(t_expr *expr, long argc, struct ex_ex *argv, struct ex_ex *optr);
 static void ex_erf(t_expr *expr, long argc, struct ex_ex *argv, struct ex_ex *optr);
 static void ex_erfc(t_expr *expr, long argc, struct ex_ex *argv, struct ex_ex *optr);
@@ -172,7 +172,7 @@ t_ex_func ex_funcs[] = {
         {"ldexp",       ex_ldexp,       2},
         {"imodf",       ex_imodf,       1},
         {"modf",        ex_modf,        1},
-#if !defined(_MSC_VER) || (_MSC_VER >= 17000)
+#if !defined(_MSC_VER) || (_MSC_VER >= 1700)
         {"asinh",       ex_asinh,       1},
         {"acosh",       ex_acosh,       1},
         {"atanh",       ex_atanh,       1},     /* hyperbolic atan */
@@ -537,9 +537,8 @@ ex_toint(t_expr *e, long int argc, struct ex_ex *argv, struct ex_ex *optr)
                 FUNC_EVAL_UNARY(left, toint, (int), optr, 0);
         }
 
-#ifdef _MSC_VER
-/* rint is now advertised as part of the microsoft SDK but my MSVC still
-doesn't find it - so here it is again. */
+#if defined _MSC_VER && (_MSC_VER < 1800)
+/* rint is not available for Visual Studio Version < Visual Studio 2013 */
 static double rint(double x)
 {
         return (floor(x + 0.5));
@@ -867,7 +866,7 @@ ex_tanh(t_expr *e, long int argc, struct ex_ex *argv, struct ex_ex *optr)
 }
 
 
-#if !defined(_MSC_VER) || (_MSC_VER >= 17000)
+#if !defined(_MSC_VER) || (_MSC_VER >= 1700)
 static void
 ex_asinh(t_expr *e, long argc, struct ex_ex *argv, struct ex_ex *optr)
 {
@@ -916,7 +915,7 @@ ex_dofact(int i)
 {
         int ret = 0;
 
-        if (i)
+        if (i > 0)
                 ret = 1;
         else
                 return (1);
@@ -1268,7 +1267,7 @@ FUNC_DEF_UNARY(ex_modf, fracmodf, (double), 1);
  */
 FUNC_DEF(ex_ldexp, ldexp, (double), (int), 1);
 
-#if !defined(_MSC_VER) || (_MSC_VER >= 17000)
+#if !defined(_MSC_VER) || (_MSC_VER >= 1700)
 /*
  * ex_cbrt - cube root
  */
